@@ -1,6 +1,6 @@
-/* $Id: hparser.c,v 2.75 2001/05/09 07:11:22 gisle Exp $
+/* $Id: hparser.c,v 2.82 2002/03/17 20:07:57 gisle Exp $
  *
- * Copyright 1999-2001, Gisle Aas
+ * Copyright 1999-2002, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
  *
  * This library is free software; you can redistribute it and/or
@@ -84,6 +84,8 @@ char *argname[] = {
     /* ARG_FLAG_FLAT_ARRAY */
 };
 
+#define CASE_SENSITIVE(p_state) \
+         ((p_state)->xml_mode || (p_state)->case_sensitive)
 
 static void flush_pending_text(PSTATE* p_state, SV* self);
 
@@ -182,11 +184,11 @@ report_event(PSTATE* p_state,
     }
 
     if (event == E_NONE)
-	goto IGNORE;
+	goto IGNORE_EVENT;
     
 #ifdef MARKED_SECTION
     if (p_state->ms == MS_IGNORE)
-	goto IGNORE;
+	goto IGNORE_EVENT;
 #endif
 
     /* tag filters */
@@ -198,7 +200,7 @@ report_event(PSTATE* p_state,
 
 	    assert(num_tokens >= 1);
 	    sv_setpvn(tagname, tokens[0].beg, tokens[0].end - tokens[0].beg);
-	    if (!p_state->xml_mode)
+	    if (!CASE_SENSITIVE(p_state))
 		sv_lower(aTHX_ tagname);
 
 	    if (p_state->ignoring_element) {
@@ -210,7 +212,7 @@ report_event(PSTATE* p_state,
 			p_state->ignoring_element = 0;
 		    }
 		}
-		goto IGNORE;
+		goto IGNORE_EVENT;
 	    }
 
 	    PERL_HASH(hash, SvPVX(tagname), SvCUR(tagname));
@@ -220,22 +222,22 @@ report_event(PSTATE* p_state,
 	    {
 		p_state->ignoring_element = newSVsv(tagname);
 		p_state->ignore_depth = 1;
-		goto IGNORE;
+		goto IGNORE_EVENT;
 	    }
 
 	    if (p_state->ignore_tags &&
 		hv_fetch_ent(p_state->ignore_tags, tagname, 0, hash))
 	    {
-		goto IGNORE;
+		goto IGNORE_EVENT;
 	    }
 	    if (p_state->report_tags &&
 		!hv_fetch_ent(p_state->report_tags, tagname, 0, hash))
 	    {
-		goto IGNORE;
+		goto IGNORE_EVENT;
 	    }
 	}
 	else if (p_state->ignoring_element) {
-	    goto IGNORE;
+	    goto IGNORE_EVENT;
 	}
     }
 
@@ -244,7 +246,7 @@ report_event(PSTATE* p_state,
 	/* event = E_DEFAULT; */
 	h = &p_state->handlers[E_DEFAULT];
 	if (!h->cb)
-	    goto IGNORE;
+	    goto IGNORE_EVENT;
     }
 
     if (SvTYPE(h->cb) != SVt_PVAV && !SvTRUE(h->cb)) {
@@ -364,7 +366,7 @@ report_event(PSTATE* p_state,
 	    if (num_tokens >= 1) {
 		arg = sv_2mortal(newSVpvn(tokens[0].beg,
 					  tokens[0].end - tokens[0].beg));
-		if (!p_state->xml_mode && argcode != ARG_TOKEN0)
+		if (!CASE_SENSITIVE(p_state) && argcode != ARG_TOKEN0)
 		    sv_lower(aTHX_ arg);
 		if (argcode == ARG_TAG && event != E_START) {
 		    char *e_type = "!##/#?#";
@@ -398,7 +400,8 @@ report_event(PSTATE* p_state,
 			    beg++; len -= 2;
 			}
 			attrval = newSVpvn(beg, len);
-			decode_entities(aTHX_ attrval, p_state->entity2char);
+			if (!p_state->attr_encoded)
+			    decode_entities(aTHX_ attrval, p_state->entity2char);
 		    }
 		    else { /* boolean */
 			if (p_state->bool_attr_val)
@@ -407,7 +410,7 @@ report_event(PSTATE* p_state,
 			    attrval = newSVsv(attrname);
 		    }
 
-		    if (!p_state->xml_mode)
+		    if (!CASE_SENSITIVE(p_state))
 			sv_lower(aTHX_ attrname);
 
 		    if (argcode == ARG_ATTR) {
@@ -440,7 +443,7 @@ report_event(PSTATE* p_state,
 		for (i = 1; i < num_tokens; i += 2) {
 		    SV* attrname = newSVpvn(tokens[i].beg,
 					    tokens[i].end-tokens[i].beg);
-		    if (!p_state->xml_mode)
+		    if (!CASE_SENSITIVE(p_state))
 			sv_lower(aTHX_ attrname);
 		    av_push(av, attrname);
 		}
@@ -538,10 +541,14 @@ report_event(PSTATE* p_state,
 
 	if ((enum argcode)*argspec == ARG_SELF && !SvROK(h->cb)) {
 	    char *method = SvPV(h->cb, my_na);
-	    perl_call_method(method, G_DISCARD | G_VOID);
+	    perl_call_method(method, G_DISCARD | G_EVAL | G_VOID);
 	}
 	else {
-	    perl_call_sv(h->cb, G_DISCARD | G_VOID);
+	    perl_call_sv(h->cb, G_DISCARD | G_EVAL | G_VOID);
+	}
+
+	if (SvTRUE(ERRSV)) {
+	    RETHROW;
 	}
 
 	FREETMPS;
@@ -551,7 +558,7 @@ report_event(PSTATE* p_state,
 	SvCUR_set(p_state->skipped_text, 0);
     return;
 
-IGNORE:
+IGNORE_EVENT:
     if (p_state->skipped_text) {
 	if (event != E_TEXT && p_state->pend_text && SvOK(p_state->pend_text))
 	    flush_pending_text(p_state, self);
@@ -1350,6 +1357,8 @@ parse(pTHX_
 	if (p_state->line)
 	    p_state->line = 1;
 	p_state->column = 0;
+	p_state->literal_mode = 0;
+	p_state->is_cdata = 0;
 	return;
     }
 
@@ -1400,7 +1409,7 @@ parse(pTHX_
 		    l++;
 		}
 
-		if (!*l) {
+		if (!*l && strNE(p_state->literal_mode, "plaintext")) {
 		    /* matched it all */
 		    token_pos_t end_token;
 		    end_token.beg = end_text + 2;
